/* -----------------------------------------------------------------------------
 *
 *	ctl2sig~
 *		scales control values (0.0 to 1.0) to signal values (-1.0 to 1.0)
 *		
 *
 *	2007 . Cooper Baker . www.cooperbaker.com
 *
 *----------------------------------------------------------------------------*/

#include <Carbon/Carbon.h>
#include <MaxAPI/MaxAPI.h>
#include <MaxAudioAPI/MaxAudioAPI.h>

void *ctl2sig_class;									// pointer to this 'class'

// ctl2sig~ structure ------------------------------------------------------ 
typedef struct _ctl2sig									// ctl2sig data structure
{ 
	t_pxobject	x_obj;									// every ctl2sig external needs this; t_pxobject struct contains object info
} t_ctl2sig; 

// prototypes ------------------------------------------------------------------ 
void *ctl2sig_new(void);								// makes a new ctl2sig~ object
void ctl2sig_free(t_ctl2sig *x);						// deletes the ctl2sig~ object
void ctl2sig_dsp(t_ctl2sig *x, t_signal **sp, short *count); // sets up dsp for ctl2sig
t_int *ctl2sig_perform(t_int *w);						// the dsp perform function
void ctl2sig_assist(t_ctl2sig *x, void *b, long m, long a, char *s);
														// inlet / outlet assistance
// main ------------------------------------------------------------------------
void main(void) 
{ 
	setup((t_messlist **)&ctl2sig_class, (method)ctl2sig_new, (method)ctl2sig_free, (short)sizeof(t_ctl2sig), 0L, 0); 
	addmess((method)ctl2sig_dsp, "dsp", A_CANT, 0);		// add dsp method
	addmess((method)ctl2sig_assist, "assist", A_CANT, 0); // assist method 
	dsp_initclass();									// init the class
	post( "" );								
	post( "cb.ctl2sig~ :: scales control values to signal values" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );
}														// post after init completes

// ctl2sig_dsp --------------------------------------------------------------------
void ctl2sig_dsp(t_ctl2sig *x, t_signal **sp, short *count)	// ctl2sig_dsp(*data struct, **signal struct, signal count )
{ 
	dsp_add( ctl2sig_perform, 4, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n, x );
}						   // 4 = number of values passed to ctl2sig_perform

// ctl2sig_perform ---------------------------------------------------------------- 
t_int *ctl2sig_perform(t_int *w)						// ctl2sig_perform runs once per signal vector
{ 
	t_float *in		= (t_float *)(w[1]);				// inlet signal vector location
	t_float *out	= (t_float *)(w[2]);				// outlet signal vector location
	int n			= (int)(w[3]);						// vector size in samples
	t_ctl2sig *x	= (t_ctl2sig *)(w[4]);				// ctl2sig struct location
	float sample	= 0;
// DSP BLOCK ///////////////////////////////////////////////////////////////////	
	while ( n-- )										// while a sample remains, do dsp ( n-- is the counter )
	{  
		sample = *in++;									// *in++/*out++ should only be done once per while loop cycle
			
		sample = ( sample * 2) - 1;						// double up and push down
		
		*out++ = sample;								// sample is used to make things clearer
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 5);										// always return w plus one more than the 2nd argument in dsp_add() 
} 

// ctl2sig_assist -----------------------------------------------------------------
void ctl2sig_assist(t_ctl2sig *x, void *b, long m, long a, char *s)
{
	switch (m)											// inlet / outlet switch
	{
		case 1:											// 1 = inlets
			switch (a)									// inlet number switch
			{
				case 0:									// inlet 0
					sprintf(s, "(signal) Thru~ Input");
					break;								// assist message
				default:
					break;
			}
			break;
		case 2:											// 2 = outlets
			switch (a)									// outlet number switch
			{
				case 0:									// outlet 0
					sprintf(s, "(signal) Thru~ Output");
					break;								// assist message
				default:
					break;
			}
			break;
	}
}

// ctl2sig_new() ------------------------------------------------------------------ 
void *ctl2sig_new(void)									// ctl2sig~ 'constructor'
{ 
	t_ctl2sig *x = (t_ctl2sig *)newobject(ctl2sig_class); // register the object with ctl2sig and save its address as *x
	dsp_setup((t_pxobject *)x, 1);						// create left inlet 
	outlet_new((t_pxobject *)x, "signal");				// create left outlet 
	return (x);											// where am i?
} 

// ctl2sig_free() -----------------------------------------------------------------
void ctl2sig_free(t_ctl2sig *x)							// ctl2sig~ 'destructor'
{
	dsp_free((t_pxobject *)x);							// ctl2sig free function
}
