/* -----------------------------------------------------------------------------
 *
 *	msp~
 *		template for msp~ externals
 *		
 *
 *	2007 . Cooper Baker . www.cooperbaker.com
 *
 *----------------------------------------------------------------------------*/

#include <Carbon/Carbon.h>
#include <MaxAPI/MaxAPI.h>
#include <MaxAudioAPI/MaxAudioAPI.h>

void *msp_class;										// pointer to this 'class'

// msp~ structure ------------------------------------------------------ 
typedef struct _msp										// msp data structure
{ 
	t_pxobject	x_obj;									// every msp external needs this; t_pxobject struct contains object info
} t_msp; 

// prototypes ------------------------------------------------------------------ 
void *msp_new(void);									// makes a new msp~ object
void msp_free(t_msp *x);								// deletes the msp~ object
void msp_dsp(t_msp *x, t_signal **sp, short *count);	// sets up dsp for msp
t_int *msp_perform(t_int *w);							// the dsp perform function
void msp_assist(t_msp *x, void *b, long m, long a, char *s);
														// inlet / outlet assistance
// main ------------------------------------------------------------------------
void main(void) 
{ 
	setup((t_messlist **)&msp_class, (method)msp_new, (method)msp_free, (short)sizeof(t_msp), 0L, 0); 
	addmess((method)msp_dsp, "dsp", A_CANT, 0);			// add dsp method
	addmess((method)msp_assist, "assist", A_CANT, 0);	// assist method 
	dsp_initclass();									// init the class
	post( "" );								
	post( "cb.msp~ :: template for msp~ externals" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );
}														// post after init completes

// msp_dsp --------------------------------------------------------------------
void msp_dsp(t_msp *x, t_signal **sp, short *count)		// msp_dsp(*data struct, **signal struct, signal count )
{														// add msp_perform to the dsp chain with 4 values
	dsp_add( msp_perform, 4, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n, x );
}						// 4 = number of values passed to msp_perform

// msp_perform ---------------------------------------------------------------- 
t_int *msp_perform(t_int *w)							// msp_perform runs once per signal vector
{ 
	t_float *in		= (t_float *)(w[1]);					// inlet signal vector location
	t_float *out	= (t_float *)(w[2]);				// outlet signal vector location
	int n			= (int)(w[3]);							// vector size in samples
	t_msp *x		= (t_msp *)(w[4]);						// msp struct location
	float sample	= 0;
// DSP BLOCK ///////////////////////////////////////////////////////////////////	
	while ( n-- )										// while a sample remains, do dsp ( n-- is the counter )
	{  
		sample = *in++;									// *in++/*out++ should only be done once per while loop cycle
		*out++ = sample;								// sample is used to make things clearer
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 5);										// always return w plus one more than the 2nd argument in dsp_add() 
} 

// msp_assist -----------------------------------------------------------------
void msp_assist(t_msp *x, void *b, long m, long a, char *s)
{
	switch (m)											// inlet / outlet switch
	{
		case 1:											// 1 = inlets
			switch (a)									// inlet number switch
			{
				case 0:									// inlet 0
					sprintf(s, "(signal) Thru~ Input");
					break;								// assist message
				default:
					break;
			}
			break;
		case 2:											// 2 = outlets
			switch (a)									// outlet number switch
			{
				case 0:									// outlet 0
					sprintf(s, "(signal) Thru~ Output");
					break;								// assist message
				default:
					break;
			}
			break;
	}
}

// msp_new() ------------------------------------------------------------------ 
void *msp_new(void)										// msp~ 'constructor'
{ 
	t_msp *x = (t_msp *)newobject(msp_class);			// register the object with msp and save its address as *x
	dsp_setup((t_pxobject *)x, 1);						// create left inlet 
	outlet_new((t_pxobject *)x, "signal");				// create left outlet 
	return (x);											// where am i?
} 

// msp_free() -----------------------------------------------------------------
void msp_free(t_msp *x)									// msp~ 'destructor'
{
	dsp_free((t_pxobject *)x);							// msp free function
}
