/* -----------------------------------------------------------------------------
 *
 *	sig2ctl~
 *		scales signal values (-1.0 to 1.0) to control values (0.0 to 1.0)
 *		
 *
 *	2007 . Cooper Baker . www.cooperbaker.com
 *
 *----------------------------------------------------------------------------*/

#include <Carbon/Carbon.h>
#include <MaxAPI/MaxAPI.h>
#include <MaxAudioAPI/MaxAudioAPI.h>

void *sig2ctl_class;									// pointer to this 'class'

// sig2ctl~ structure ------------------------------------------------------ 
typedef struct _sig2ctl									// sig2ctl data structure
{ 
	t_pxobject	x_obj;									// every sig2ctl external needs this; t_pxobject struct contains object info
} t_sig2ctl; 

// prototypes ------------------------------------------------------------------ 
void *sig2ctl_new(void);								// makes a new sig2ctl~ object
void sig2ctl_free(t_sig2ctl *x);						// deletes the sig2ctl~ object
void sig2ctl_dsp(t_sig2ctl *x, t_signal **sp, short *count);// sets up dsp for sig2ctl
t_int *sig2ctl_perform(t_int *w);						// the dsp perform function
void sig2ctl_assist(t_sig2ctl *x, void *b, long m, long a, char *s);
														// inlet / outlet assistance
// main ------------------------------------------------------------------------
void main(void) 
{ 
	setup((t_messlist **)&sig2ctl_class, (method)sig2ctl_new, (method)sig2ctl_free, (short)sizeof(t_sig2ctl), 0L, 0); 
	addmess((method)sig2ctl_dsp, "dsp", A_CANT, 0);		// add dsp method
	addmess((method)sig2ctl_assist, "assist", A_CANT, 0);// assist method 
	dsp_initclass();									// init the class
	post( "" );								
	post( "cb.sig2ctl~ :: scales signal values to control values" );
	post( "2007 . Cooper Baker . www.cooperbaker.com" );
	post( "" );
}														// post after init completes

// sig2ctl_dsp --------------------------------------------------------------------
void sig2ctl_dsp(t_sig2ctl *x, t_signal **sp, short *count)	// sig2ctl_dsp(*data struct, **signal struct, signal count )
{ 
	dsp_add( sig2ctl_perform, 4, sp[0]->s_vec, sp[1]->s_vec, sp[0]->s_n, x );
}						   // 4 = number of values passed to sig2ctl_perform

// sig2ctl_perform ---------------------------------------------------------------- 
t_int *sig2ctl_perform(t_int *w)						// sig2ctl_perform runs once per signal vector
{ 
	t_float *in		= (t_float *)(w[1]);				// inlet signal vector location
	t_float *out	= (t_float *)(w[2]);				// outlet signal vector location
	int n			= (int)(w[3]);						// vector size in samples
	t_sig2ctl *x	= (t_sig2ctl *)(w[4]);				// sig2ctl struct location
	float sample	= 0;
// DSP BLOCK ///////////////////////////////////////////////////////////////////	
	while ( n-- )										// while a sample remains, do dsp ( n-- is the counter )
	{  
		sample = *in++;									// *in++/*out++ should only be done once per while loop cycle
		
		sample = ( sample + 1 ) * 0.5;					// pull up and halve
		
		*out++ = sample;								// sample is used to make things clearer
	}
// END DSP BLOCK ///////////////////////////////////////////////////////////////	
	
	return (w + 5);										// always return w plus one more than the 2nd argument in dsp_add() 
} 

// sig2ctl_assist -----------------------------------------------------------------
void sig2ctl_assist(t_sig2ctl *x, void *b, long m, long a, char *s)
{
	switch (m)											// inlet / outlet switch
	{
		case 1:											// 1 = inlets
			switch (a)									// inlet number switch
			{
				case 0:									// inlet 0
					sprintf(s, "(signal) input");
					break;								// assist message
				default:
					break;
			}
			break;
		case 2:											// 2 = outlets
			switch (a)									// outlet number switch
			{
				case 0:									// outlet 0
					sprintf(s, "(signal) output");
					break;								// assist message
				default:
					break;
			}
			break;
	}
}

// sig2ctl_new() ------------------------------------------------------------------ 
void *sig2ctl_new(void)									// sig2ctl~ 'constructor'
{ 
	t_sig2ctl *x = (t_sig2ctl *)newobject(sig2ctl_class);// register the object with sig2ctl and save its address as *x
	dsp_setup((t_pxobject *)x, 1);						// create left inlet 
	outlet_new((t_pxobject *)x, "signal");				// create left outlet 
	return (x);											// where am i?
} 

// sig2ctl_free() -----------------------------------------------------------------
void sig2ctl_free(t_sig2ctl *x)							// sig2ctl~ 'destructor'
{
	dsp_free((t_pxobject *)x);							// sig2ctl free function
}
